@@ -1,7 +1,5 @@
 from hazelcast.serialization.bits import *
 from hazelcast.protocol.client_message import ClientMessage
-from hazelcast.protocol.custom_codec import *
-from hazelcast.util import ImmutableLazyDataList
 from hazelcast.protocol.codec.atomic_reference_message_type import *
 
 REQUEST_TYPE = ATOMICREFERENCE_CONTAINS
@@ -37,6 +35,3 @@ def decode_response(client_message, to_object=None):
     parameters = dict(response=None)
     parameters['response'] = client_message.read_bool()
     return parameters
-
-
-
