@@ -1,7 +1,5 @@
 from hazelcast.serialization.bits import *
 from hazelcast.protocol.client_message import ClientMessage
-from hazelcast.protocol.custom_codec import *
-from hazelcast.util import ImmutableLazyDataList
 from hazelcast.protocol.codec.atomic_reference_message_type import *
 
 REQUEST_TYPE = ATOMICREFERENCE_GETANDSET
@@ -35,10 +33,6 @@ def encode_request(name, new_value):
 def decode_response(client_message, to_object=None):
     """ Decode response from client message"""
     parameters = dict(response=None)
-    response=None
     if not client_message.read_bool():
         parameters['response'] = to_object(client_message.read_data())
     return parameters
-
-
-
