@@ -1,8 +1,8 @@
-from hazelcast.serialization.bits import *
 from hazelcast.protocol.client_message import ClientMessage
 from hazelcast.protocol.custom_codec import *
 from hazelcast.util import ImmutableLazyDataList
 from hazelcast.protocol.codec.client_message_type import *
+from hazelcast.six.moves import range
 
 REQUEST_TYPE = CLIENT_GETDISTRIBUTEDOBJECTS
 RESPONSE_TYPE = 110
@@ -29,11 +29,8 @@ def decode_response(client_message, to_object=None):
     parameters = dict(response=None)
     response_size = client_message.read_int()
     response = []
-    for response_index in xrange(0, response_size):
-        response_item = DistributedObjectInfoCodec.decode(client_message, to_object)
+    for _ in range(0, response_size):
+        response_item = DistributedObjectInfoCodec.decode(client_message)
         response.append(response_item)
     parameters['response'] = ImmutableLazyDataList(response, to_object)
     return parameters
-
-
-
