@@ -1,7 +1,5 @@
 from hazelcast.serialization.bits import *
 from hazelcast.protocol.client_message import ClientMessage
-from hazelcast.protocol.custom_codec import *
-from hazelcast.util import ImmutableLazyDataList
 from hazelcast.protocol.codec.transactional_map_message_type import *
 
 REQUEST_TYPE = TRANSACTIONALMAP_SET
@@ -35,6 +33,3 @@ def encode_request(name, txn_id, thread_id, key, value):
 
 
 # Empty decode_response(client_message), this message has no parameters to decode
-
-
-
