@@ -12,12 +12,16 @@ class MultiMap(Proxy):
     """
     A specialized map whose keys can be associated with multiple values.
     """
+    def __init__(self, client, service_name, name):
+        super(MultiMap, self).__init__(client, service_name, name)
+        self.reference_id_generator = self._client.lock_reference_id_generator
+
     def add_entry_listener(self, include_value=False, key=None, added_func=None, removed_func=None, clear_all_func=None):
         """
         Adds an entry listener for this multimap. The listener will be notified for all multimap add/remove/clear-all
         events.
 
-        :param include_value: (bool), whether received events include an old value or not (optional).
+        :param include_value: (bool), whether received event should include the value or not (optional).
         :param key: (object), key for filtering the events (optional).
         :param added_func: Function to be called when an entry is added to map (optional).
         :param removed_func: Function to be called when an entry is removed_func from map (optional).
@@ -26,12 +30,11 @@ class MultiMap(Proxy):
         """
         if key:
             key_data = self._to_data(key)
-            request = multi_map_add_entry_listener_to_key_codec.encode_request(name=self.name, key=key_data,
-                                                                               include_value=include_value,
-                                                                               local_only=False)
+            request = multi_map_add_entry_listener_to_key_codec.encode_request(
+                name=self.name, key=key_data, include_value=include_value, local_only=False)
         else:
-            request = multi_map_add_entry_listener_codec.encode_request(name=self.name, include_value=include_value,
-                                                                        local_only=False)
+            request = multi_map_add_entry_listener_codec.encode_request(
+                name=self.name, include_value=include_value, local_only=False)
 
         def handle_event_entry(**_kwargs):
             event = EntryEvent(self._to_object, **_kwargs)
@@ -42,11 +45,10 @@ class MultiMap(Proxy):
             elif event.event_type == EntryEventType.clear_all and clear_all_func:
                 clear_all_func(event)
 
-        return self._start_listening(request,
-                                     lambda m: multi_map_add_entry_listener_codec.handle(m,
-                                                                                         handle_event_entry),
-                                     lambda r: multi_map_add_entry_listener_codec.decode_response(r)[
-                                         'response'])
+        return self._register_listener(
+            request, lambda r: multi_map_add_entry_listener_codec.decode_response(r)['response'],
+            lambda reg_id: multi_map_remove_entry_listener_codec.encode_request(self.name, reg_id),
+            lambda m: multi_map_add_entry_listener_codec.handle(m, handle_event_entry))
 
     def contains_key(self, key):
         """
@@ -152,7 +154,8 @@ class MultiMap(Proxy):
         """
         check_not_none(key, "key can't be None")
         key_data = self._to_data(key)
-        return self._encode_invoke_on_key(multi_map_force_unlock_codec, key_data, key=key_data)
+        return self._encode_invoke_on_key(multi_map_force_unlock_codec, key_data, key=key_data,
+                                          reference_id=self.reference_id_generator.get_and_increment())
 
     def key_set(self):
         """
@@ -186,7 +189,8 @@ class MultiMap(Proxy):
         check_not_none(key, "key can't be None")
         key_data = self._to_data(key)
         return self._encode_invoke_on_key(multi_map_lock_codec, key_data, key=key_data,
-                                          thread_id=thread_id(), ttl=to_millis(lease_time))
+                                          thread_id=thread_id(), ttl=to_millis(lease_time),
+                                          reference_id=self.reference_id_generator.get_and_increment())
 
     def remove(self, key, value):
         """
@@ -251,8 +255,7 @@ class MultiMap(Proxy):
         :param registration_id: (str), id of registered listener.
         :return: (bool), ``true`` if registration is removed, ``false`` otherwise.
         """
-        return self._stop_listening(registration_id,
-                                    lambda i: multi_map_remove_entry_listener_codec.encode_request(self.name, i))
+        return self._deregister_listener(registration_id)
 
     def size(self):
         """
@@ -310,7 +313,8 @@ class MultiMap(Proxy):
         key_data = self._to_data(key)
         return self._encode_invoke_on_key(multi_map_try_lock_codec, key_data, key=key_data,
                                           thread_id=thread_id(), lease=to_millis(lease_time),
-                                          timeout=to_millis(timeout))
+                                          timeout=to_millis(timeout),
+                                          reference_id=self.reference_id_generator.get_and_increment())
 
     def unlock(self, key):
         """
@@ -324,4 +328,5 @@ class MultiMap(Proxy):
         check_not_none(key, "key can't be None")
         key_data = self._to_data(key)
         return self._encode_invoke_on_key(multi_map_unlock_codec, key_data, key=key_data,
-                                          thread_id=thread_id())
+                                          thread_id=thread_id(),
+                                          reference_id=self.reference_id_generator.get_and_increment())
