@@ -1,8 +1,11 @@
 import struct
 
-from api import *
-from bits import *
+from hazelcast.serialization.api import *
+from hazelcast.serialization.bits import *
 from hazelcast.serialization.data import Data
+from hazelcast import six
+from hazelcast.six.moves import range
+from hazelcast.config import ClientProperties
 
 
 class _ObjectDataInput(ObjectDataInput):
@@ -12,6 +15,7 @@ class _ObjectDataInput(ObjectDataInput):
         self._is_big_endian = is_big_endian
         self._pos = offset
         self._size = len(buff)
+        self._respect_bytearrays = False if serialization_service is None else serialization_service.properties.get_bool(ClientProperties.SERIALIZATION_INPUT_RETURNS_BYTEARRAY)
         # Local cache struct formats according to endianness
         self._FMT_INT8 = FMT_BE_INT8 if self._is_big_endian else FMT_LE_INT8
         self._FMT_UINT8 = FMT_BE_UINT8 if self._is_big_endian else FMT_LE_UINT8
@@ -52,7 +56,7 @@ class _ObjectDataInput(ObjectDataInput):
 
     def read_char(self, position=None):
         char_ord = self.read_short(position)
-        return unichr(char_ord)
+        return six.unichr(char_ord)
 
     def read_short(self, position=None):
         self._check_available(self._pos, SHORT_SIZE_IN_BYTES)
@@ -83,7 +87,7 @@ class _ObjectDataInput(ObjectDataInput):
         if length == NULL_ARRAY_LENGTH:
             return None
         result = bytearray()
-        for i in xrange(0, length):
+        for i in range(0, length):
             _first_byte = self.read_byte() & 0xFF
             b = _first_byte >> 4
             if 0 <= b <= 7:
@@ -108,7 +112,11 @@ class _ObjectDataInput(ObjectDataInput):
         result = bytearray(length)
         if length > 0:
             self.read_into(result, 0, length)
-        return [x for x in result]
+
+        if self._respect_bytearrays:
+            return result
+
+        return list(result)
 
     def read_boolean_array(self):
         return self._read_array_fnc(self.read_boolean)
@@ -174,7 +182,7 @@ class _ObjectDataInput(ObjectDataInput):
         if length == NULL_ARRAY_LENGTH:
             return None
         if length > 0:
-            return [read_item_fnc() for _ in xrange(0, length)]
+            return [read_item_fnc() for _ in range(0, length)]
         return []
 
     def __repr__(self):
